------------------------------------------------------------------------------
-- Test HNSW index creation
------------------------------------------------------------------------------

-- Validate that index creation works with a small number of vectors
\ir utils/small_world_array_unlogged.sql
\ir utils/sift1k_array_unlogged.sql

-- Validate that creating a secondary index works
CREATE INDEX ON sift_base1k USING lantern_hnsw (v) WITH (dim=128, M=6);
SELECT * FROM ldb_get_indexes('sift_base1k');
SELECT _lantern_internal.validate_index('sift_base1k_v_idx', false);

-- Validate that index creation works with a larger number of vectors
\ir utils/sift10k_array_unlogged.sql
SET lantern.pgvector_compat=FALSE;

CREATE INDEX hnsw_idx ON sift_base10k USING lantern_hnsw (v dist_l2sq_ops) WITH (M=2, ef_construction=10, ef=4, dim=128);
SELECT v AS v4444 FROM sift_base10k WHERE id = 4444 \gset
EXPLAIN (COSTS FALSE) SELECT * FROM sift_base10k order by v <?> :'v4444' LIMIT 10;
SELECT _lantern_internal.validate_index('hnsw_idx', false);

--- Validate that M values inside the allowed range [2, 128] do not throw an error

CREATE INDEX ON small_world USING lantern_hnsw (v) WITH (M=2);
CREATE INDEX ON small_world USING lantern_hnsw (v) WITH (M=128);

---- Validate that M values outside the allowed range [2, 128] throw an error
\set ON_ERROR_STOP off
CREATE INDEX ON small_world USING lantern_hnsw (v) WITH (M=1);
CREATE INDEX ON small_world USING lantern_hnsw (v) WITH (M=129);
\set ON_ERROR_STOP on

-- Validate index dimension inference
CREATE UNLOGGED TABLE small_world4 (
    id varchar(3),
    vector real[]
);
-- If the first row is NULL we do not infer a dimension
\set ON_ERROR_STOP off
CREATE INDEX ON small_world4 USING lantern_hnsw (vector) WITH (M=14, ef=22, ef_construction=2);
begin;
INSERT INTO small_world4 (id, vector) VALUES
('000', NULL),
('001', '{1,0,0,1}');
CREATE INDEX ON small_world4 USING lantern_hnsw (vector) WITH (M=14, ef=22, ef_construction=2);
rollback;
\set ON_ERROR_STOP on

INSERT INTO small_world4 (id, vector) VALUES
('000', '{1,0,0,0}'),
('001', '{1,0,0,1}'),
('010', '{1,0,1,0}'),
('011', '{1,0,1,1}'),
('100', '{1,1,0,0}'),
('101', '{1,1,0,1}'),
('110', '{1,1,1,0}'),
('111', '{1,1,1,1}');
CREATE INDEX small_world4_hnsw_idx ON small_world4 USING lantern_hnsw (vector) WITH (M=14, ef=22, ef_construction=2);
SELECT * FROM ldb_get_indexes('small_world4');
-- the index will not allow changing the dimension of a vector element
\set ON_ERROR_STOP off
UPDATE small_world4 SET vector = '{0,0,0}' WHERE id = '000';
UPDATE small_world4 SET vector = '{0,0,0}' WHERE id = '001';
\set ON_ERROR_STOP on

INSERT INTO small_world4 (id, vector) VALUES
('000', '{1,0,0,0}'),
('001', '{1,0,0,1}'),
('010', '{1,0,1,0}');

SELECT _lantern_internal.validate_index('small_world4_hnsw_idx', false);

-- without the index, I can change the dimension of a vector element
DROP INDEX small_world4_hnsw_idx;
UPDATE small_world4 SET vector = '{0,0,0}' WHERE id = '001';
-- but then, I cannot create the same dimension-inferred index
\set ON_ERROR_STOP off
CREATE INDEX ON small_world4 USING lantern_hnsw (vector) WITH (M=14, ef=22, ef_construction=2);
\set ON_ERROR_STOP on
